package application_v1

import (
	"github.com/gin-gonic/gin"
	"github.com/iVampireSP/pkg/page"
	"leafdev.top/Ecosystem/recommender/internal/entity"
	"leafdev.top/Ecosystem/recommender/internal/handler/http/request"
	"leafdev.top/Ecosystem/recommender/internal/handler/http/response"
	"leafdev.top/Ecosystem/recommender/internal/service/application"
	"leafdev.top/Ecosystem/recommender/internal/service/auth"
	"leafdev.top/Ecosystem/recommender/internal/service/category"
	"leafdev.top/Ecosystem/recommender/internal/service/post"
	"net/http"
)

type CategoryController struct {
	authService        *auth.Service
	applicationService *application.Service
	postService        *post.Service
	categoryService    *category.Service
}

func NewCategoryController(
	authService *auth.Service,
	applicationService *application.Service,
	postService *post.Service,
	categoryService *category.Service,
) *CategoryController {
	return &CategoryController{
		authService,
		applicationService,
		postService,
		categoryService,
	}
}

// List godoc
// @Summary      列出分类
// @Description  列出分类
// @Tags         application_api
// @Accept       json
// @Produce      json
// @Security     ApiKeyAuth
// @Success      200  {object}  response.ResponseBody{data=page.PagedResult[entity.Category]}
// @Failure      400  {object}  response.ResponseBody
// @Router       /applications/v1/categories [get]
func (cc *CategoryController) List(c *gin.Context) {
	app, err := cc.authService.GetApplication(c)
	if err != nil {
		response.Ctx(c).Error(err).Status(http.StatusBadRequest).Send()
		return
	}

	var pagedResult = page.NewPagedResult[*entity.Category]()

	err = cc.categoryService.ListCategories(c, pagedResult, app)

	response.Ctx(c).Data(pagedResult.Output()).Send()
	return
}

// Save godoc
// @Summary      新建分类
// @Description  新建分类
// @Tags         application_api
// @Accept       json
// @Produce      json
// @Security     ApiKeyAuth
// @Param        CategorySaveRequest  body  request.CategorySaveRequest  true  "body"
// @Success      200  {object}  response.ResponseBody{data=entity.Application}
// @Failure      400  {object}  response.ResponseBody
// @Router       /applications/v1/categories [post]
func (cc *CategoryController) Save(c *gin.Context) {
	app, err := cc.authService.GetApplication(c)
	if err != nil {
		response.Ctx(c).Error(err).Status(http.StatusBadRequest).Send()
		return
	}

	var categorySaveRequest = &request.CategorySaveRequest{}
	if err := c.ShouldBindJSON(categorySaveRequest); err != nil {
		response.Ctx(c).Error(err).Status(http.StatusBadRequest).Send()
		return
	}

	var categoryEntity = &entity.Category{
		Name:          categorySaveRequest.Name,
		ApplicationId: app.Id,
	}

	err = cc.categoryService.CreateCategory(c, categoryEntity, app)

	response.Ctx(c).Error(err).Data(categoryEntity).Send()
	return
}

// Delete godoc
// @Summary      删除分类
// @Description  删除分类
// @Tags         application_api
// @Accept       json
// @Produce      json
// @Security     ApiKeyAuth
// @Param        CategoryIdRequest path request.CategoryIdRequest true "CategoryIdRequest"
// @Success      200  {object}  response.ResponseBody{data=entity.Category}
// @Failure      400  {object}  response.ResponseBody
// @Router       /applications/v1/categories/{category_id} [delete]
func (cc *CategoryController) Delete(c *gin.Context) {
	app, err := cc.authService.GetApplication(c)
	if err != nil {
		response.Ctx(c).Error(err).Status(http.StatusBadRequest).Send()
		return
	}

	var categoryIdRequest = &request.CategoryIdRequest{}
	if err := c.ShouldBindUri(categoryIdRequest); err != nil {
		response.Ctx(c).Error(err).Status(http.StatusBadRequest).Send()
		return
	}

	categoryEntity, err := cc.categoryService.GetCategoryById(c, categoryIdRequest.CategoryId)
	if err != nil {
		response.Ctx(c).Error(err).Status(http.StatusBadRequest).Send()
		return
	}

	if categoryEntity.ApplicationId != app.Id {
		response.Ctx(c).Status(http.StatusForbidden).Send()
		return
	}

	err = cc.categoryService.DeleteCategory(c, categoryEntity)
	if err != nil {
		response.Ctx(c).Error(err).Send()
	}

	response.Ctx(c).Status(http.StatusNoContent).Send()

	return
}

// Get godoc
// @Summary      获取 Post
// @Description  获取 Post
// @Tags         application_api
// @Accept       json
// @Produce      json
// @Security     ApiKeyAuth
// @Param        CategoryIdRequest path request.CategoryIdRequest true "CategoryIdRequest"
// @Success      200  {object}  response.ResponseBody{data=entity.Category}
// @Failure      400  {object}  response.ResponseBody
// @Router       /applications/v1/categories/{category_id} [get]
func (cc *CategoryController) Get(c *gin.Context) {
	app, err := cc.authService.GetApplication(c)
	if err != nil {
		response.Ctx(c).Error(err).Status(http.StatusBadRequest).Send()
		return
	}

	var categoryIdRequest = &request.CategoryIdRequest{}
	if err := c.ShouldBindUri(categoryIdRequest); err != nil {
		response.Ctx(c).Error(err).Status(http.StatusBadRequest).Send()
		return
	}

	categoryEntity, err := cc.categoryService.GetCategoryById(c, categoryIdRequest.CategoryId)
	if err != nil {
		response.Ctx(c).Error(err).Status(http.StatusBadRequest).Send()
		return
	}

	if categoryEntity.ApplicationId != app.Id {
		response.Ctx(c).Status(http.StatusForbidden).Send()
		return
	}

	response.Ctx(c).Data(categoryEntity).Send()

	return
}
